@@ -1,5 +1,7 @@
-use criterion::{criterion_group, criterion_main, BatchSize, Criterion};
+use std::sync::atomic::{AtomicUsize, Ordering};
 use std::time::Duration;
+
+use criterion::{criterion_group, criterion_main, BatchSize, Criterion};
 use tokio::runtime::Runtime;
 use tokio::task;
 
@@ -10,10 +12,10 @@ fn collect_gc_twice(lua: &Lua) {
     lua.gc_collect().unwrap();
 }
 
-fn create_table(c: &mut Criterion) {
+fn table_create_empty(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("create [table empty]", |b| {
+    c.bench_function("table [create empty]", |b| {
         b.iter_batched(
             || collect_gc_twice(&lua),
             |_| {
@@ -24,35 +26,33 @@ fn create_table(c: &mut Criterion) {
     });
 }
 
-fn create_array(c: &mut Criterion) {
+fn table_create_array(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("create [array] 10", |b| {
+    c.bench_function("table [create array]", |b| {
         b.iter_batched(
             || collect_gc_twice(&lua),
             |_| {
-                let table = lua.create_table().unwrap();
-                for i in 1..=10 {
-                    table.set(i, i).unwrap();
-                }
+                lua.create_sequence_from(1..=10).unwrap();
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn create_string_table(c: &mut Criterion) {
+fn table_create_hash(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("create [table string] 10", |b| {
+    c.bench_function("table [create hash]", |b| {
         b.iter_batched(
             || collect_gc_twice(&lua),
             |_| {
-                let table = lua.create_table().unwrap();
-                for &s in &["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"] {
-                    let s = lua.create_string(s).unwrap();
-                    table.set(s.clone(), s).unwrap();
-                }
+                lua.create_table_from(
+                    ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
+                        .into_iter()
+                        .map(|s| (s, s)),
+                )
+                .unwrap();
             },
             BatchSize::SmallInput,
         );
@@ -62,17 +62,15 @@ fn create_string_table(c: &mut Criterion) {
 fn table_get_set(c: &mut Criterion) {
     let lua = Lua::new();
 
-    let table = lua.create_table().unwrap();
-
-    c.bench_function("table raw_get and raw_set [10]", |b| {
+    c.bench_function("table [get and set]", |b| {
         b.iter_batched(
             || {
                 collect_gc_twice(&lua);
-                table.clear().unwrap();
+                lua.create_table().unwrap()
             },
-            |_| {
-                for (i, &s) in ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
-                    .iter()
+            |table| {
+                for (i, s) in ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
+                    .into_iter()
                     .enumerate()
                 {
                     table.raw_set(s, i).unwrap();
@@ -87,7 +85,7 @@ fn table_get_set(c: &mut Criterion) {
 fn table_traversal_pairs(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("table traversal [pairs]", |b| {
+    c.bench_function("table [traversal pairs]", |b| {
         b.iter_batched(
             || lua.globals(),
             |globals| {
@@ -103,7 +101,7 @@ fn table_traversal_pairs(c: &mut Criterion) {
 fn table_traversal_for_each(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("table traversal [for_each]", |b| {
+    c.bench_function("table [traversal for_each]", |b| {
         b.iter_batched(
             || lua.globals(),
             |globals| globals.for_each::<String, LuaValue>(|_k, _v| Ok(())),
@@ -117,7 +115,7 @@ fn table_traversal_sequence(c: &mut Criterion) {
 
     let table = lua.create_sequence_from(1..1000).unwrap();
 
-    c.bench_function("table traversal [sequence]", |b| {
+    c.bench_function("table [traversal sequence]", |b| {
         b.iter_batched(
             || table.clone(),
             |table| {
@@ -130,236 +128,255 @@ fn table_traversal_sequence(c: &mut Criterion) {
     });
 }
 
-fn create_function(c: &mut Criterion) {
+fn function_create(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("create [function] 10", |b| {
+    c.bench_function("function [create Rust]", |b| {
         b.iter_batched(
             || collect_gc_twice(&lua),
             |_| {
-                for i in 0..10 {
-                    lua.create_function(move |_, ()| Ok(i)).unwrap();
-                }
+                lua.create_function(|_, ()| Ok(123)).unwrap();
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn call_lua_function(c: &mut Criterion) {
+fn function_call_sum(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("call Lua function [sum] 3 10", |b| {
-        b.iter_batched_ref(
-            || {
-                collect_gc_twice(&lua);
-                lua.load("function(a, b, c) return a + b + c end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
-            },
-            |function| {
-                for i in 0..10 {
-                    let _result: i64 = function.call((i, i + 1, i + 2)).unwrap();
-                }
-            },
-            BatchSize::SmallInput,
-        );
-    });
-}
-
-fn call_sum_callback(c: &mut Criterion) {
-    let lua = Lua::new();
-    let callback = lua
-        .create_function(|_, (a, b, c): (i64, i64, i64)| Ok(a + b + c))
+    let sum = lua
+        .create_function(|_, (a, b, c): (i64, i64, i64)| Ok(a + b - c))
         .unwrap();
-    lua.globals().set("callback", callback).unwrap();
 
-    c.bench_function("call Rust callback [sum] 3 10", |b| {
-        b.iter_batched_ref(
-            || {
-                collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do callback(i, i+1, i+2) end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
-            },
-            |function| {
-                function.call::<_, ()>(()).unwrap();
+    c.bench_function("function [call Rust sum]", |b| {
+        b.iter_batched(
+            || collect_gc_twice(&lua),
+            |_| {
+                assert_eq!(sum.call::<_, i64>((10, 20, 30)).unwrap(), 0);
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn call_async_sum_callback(c: &mut Criterion) {
-    let options = LuaOptions::new().thread_pool_size(1024);
-    let lua = Lua::new_with(LuaStdLib::ALL_SAFE, options).unwrap();
-    let callback = lua
-        .create_async_function(|_, (a, b, c): (i64, i64, i64)| async move {
-            task::yield_now().await;
-            Ok(a + b + c)
-        })
-        .unwrap();
-    lua.globals().set("callback", callback).unwrap();
-
-    c.bench_function("call async Rust callback [sum] 3 10", |b| {
-        let rt = Runtime::new().unwrap();
-        b.to_async(rt).iter_batched(
-            || {
-                collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do callback(i, i+1, i+2) end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
-            },
-            |function| async move {
-                function.call_async::<_, ()>(()).await.unwrap();
-            },
-            BatchSize::SmallInput,
-        );
-    });
-}
-
-fn call_concat_callback(c: &mut Criterion) {
+fn function_call_lua_sum(c: &mut Criterion) {
     let lua = Lua::new();
-    let callback = lua
+
+    let sum = lua
+        .load("function(a, b, c) return a + b - c end")
+        .eval::<LuaFunction>()
+        .unwrap();
+
+    c.bench_function("function [call Lua sum]", |b| {
+        b.iter_batched(
+            || collect_gc_twice(&lua),
+            |_| {
+                assert_eq!(sum.call::<_, i64>((10, 20, 30)).unwrap(), 0);
+            },
+            BatchSize::SmallInput,
+        );
+    });
+}
+
+fn function_call_concat(c: &mut Criterion) {
+    let lua = Lua::new();
+
+    let concat = lua
         .create_function(|_, (a, b): (LuaString, LuaString)| {
             Ok(format!("{}{}", a.to_str()?, b.to_str()?))
         })
         .unwrap();
-    lua.globals().set("callback", callback).unwrap();
+    let i = AtomicUsize::new(0);
 
-    c.bench_function("call Rust callback [concat string] 10", |b| {
-        b.iter_batched_ref(
+    c.bench_function("function [call Rust concat string]", |b| {
+        b.iter_batched(
             || {
                 collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do callback('a', tostring(i)) end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
+                i.fetch_add(1, Ordering::Relaxed)
             },
-            |function| {
-                function.call::<_, ()>(()).unwrap();
+            |i| {
+                assert_eq!(
+                    concat.call::<_, LuaString>(("num:", i)).unwrap(),
+                    format!("num:{i}")
+                );
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn create_registry_values(c: &mut Criterion) {
+fn function_call_lua_concat(c: &mut Criterion) {
     let lua = Lua::new();
 
-    c.bench_function("create [registry value] 10", |b| {
+    let concat = lua
+        .load("function(a, b) return a..b end")
+        .eval::<LuaFunction>()
+        .unwrap();
+    let i = AtomicUsize::new(0);
+
+    c.bench_function("function [call Lua concat string]", |b| {
         b.iter_batched(
-            || collect_gc_twice(&lua),
-            |_| {
-                for _ in 0..10 {
-                    lua.create_registry_value(lua.pack(true).unwrap()).unwrap();
-                }
-                lua.expire_registry_values();
+            || {
+                collect_gc_twice(&lua);
+                i.fetch_add(1, Ordering::Relaxed)
+            },
+            |i| {
+                assert_eq!(
+                    concat.call::<_, LuaString>(("num:", i)).unwrap(),
+                    format!("num:{i}")
+                );
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn create_userdata(c: &mut Criterion) {
-    struct UserData(i64);
+fn function_async_call_sum(c: &mut Criterion) {
+    let options = LuaOptions::new().thread_pool_size(1024);
+    let lua = Lua::new_with(LuaStdLib::ALL_SAFE, options).unwrap();
+
+    let sum = lua
+        .create_async_function(|_, (a, b, c): (i64, i64, i64)| async move {
+            task::yield_now().await;
+            Ok(a + b - c)
+        })
+        .unwrap();
+
+    c.bench_function("function [async call Rust sum]", |b| {
+        let rt = Runtime::new().unwrap();
+        b.to_async(rt).iter_batched(
+            || collect_gc_twice(&lua),
+            |_| async {
+                assert_eq!(sum.call_async::<_, i64>((10, 20, 30)).await.unwrap(), 0);
+            },
+            BatchSize::SmallInput,
+        );
+    });
+}
+
+fn registry_value_create(c: &mut Criterion) {
+    let lua = Lua::new();
+    lua.gc_stop();
+
+    c.bench_function("registry value [create]", |b| {
+        b.iter_batched(
+            || collect_gc_twice(&lua),
+            |_| lua.create_registry_value("hello").unwrap(),
+            BatchSize::SmallInput,
+        );
+    });
+}
+
+fn userdata_create(c: &mut Criterion) {
+    struct UserData(#[allow(unused)] i64);
     impl LuaUserData for UserData {}
 
     let lua = Lua::new();
 
-    c.bench_function("create [table userdata] 10", |b| {
+    c.bench_function("userdata [create]", |b| {
         b.iter_batched(
             || collect_gc_twice(&lua),
             |_| {
-                let table: LuaTable = lua.create_table().unwrap();
-                for i in 1..11 {
-                    table.set(i, UserData(i)).unwrap();
-                }
+                lua.create_userdata(UserData(123)).unwrap();
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn call_userdata_index(c: &mut Criterion) {
-    struct UserData(i64);
+fn userdata_call_index(c: &mut Criterion) {
+    struct UserData(#[allow(unused)] i64);
     impl LuaUserData for UserData {
         fn add_methods<'lua, M: LuaUserDataMethods<'lua, Self>>(methods: &mut M) {
-            methods.add_meta_method(LuaMetaMethod::Index, move |_, _, index: String| Ok(index));
+            methods.add_meta_method(LuaMetaMethod::Index, move |_, _, key: LuaString| Ok(key));
         }
     }
 
     let lua = Lua::new();
-    lua.globals().set("userdata", UserData(10)).unwrap();
+    let ud = lua.create_userdata(UserData(123)).unwrap();
+    let index = lua
+        .load("function(ud) return ud.test end")
+        .eval::<LuaFunction>()
+        .unwrap();
 
-    c.bench_function("call [userdata index] 10", |b| {
-        b.iter_batched_ref(
-            || {
-                collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do local v = userdata.test end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
-            },
-            |function| {
-                function.call::<_, ()>(()).unwrap();
+    c.bench_function("userdata [call index]", |b| {
+        b.iter_batched(
+            || collect_gc_twice(&lua),
+            |_| {
+                assert_eq!(index.call::<_, LuaString>(&ud).unwrap(), "test");
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn call_userdata_method(c: &mut Criterion) {
+fn userdata_call_method(c: &mut Criterion) {
     struct UserData(i64);
     impl LuaUserData for UserData {
         fn add_methods<'lua, M: LuaUserDataMethods<'lua, Self>>(methods: &mut M) {
-            methods.add_method("method", |_, this, ()| Ok(this.0));
+            methods.add_method("add", |_, this, i: i64| Ok(this.0 + i));
         }
     }
 
     let lua = Lua::new();
-    lua.globals().set("userdata", UserData(10)).unwrap();
+    let ud = lua.create_userdata(UserData(123)).unwrap();
+    let method = lua
+        .load("function(ud, i) return ud:add(i) end")
+        .eval::<LuaFunction>()
+        .unwrap();
+    let i = AtomicUsize::new(0);
 
-    c.bench_function("call [userdata method] 10", |b| {
-        b.iter_batched_ref(
+    c.bench_function("userdata [call method]", |b| {
+        b.iter_batched(
             || {
                 collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do userdata:method() end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
+                i.fetch_add(1, Ordering::Relaxed)
             },
-            |function| {
-                function.call::<_, ()>(()).unwrap();
+            |i| {
+                assert_eq!(method.call::<_, usize>((&ud, i)).unwrap(), 123 + i);
             },
             BatchSize::SmallInput,
         );
     });
 }
 
-fn call_async_userdata_method(c: &mut Criterion) {
-    struct UserData(String);
-
+fn userdata_async_call_method(c: &mut Criterion) {
+    struct UserData(i64);
     impl LuaUserData for UserData {
         fn add_methods<'lua, M: LuaUserDataMethods<'lua, Self>>(methods: &mut M) {
-            methods.add_async_method("method", |_, this, ()| async move { Ok(this.0.clone()) });
+            methods.add_async_method("add", |_, this, i: i64| async move {
+                task::yield_now().await;
+                Ok(this.0 + i)
+            });
         }
     }
 
     let options = LuaOptions::new().thread_pool_size(1024);
     let lua = Lua::new_with(LuaStdLib::ALL_SAFE, options).unwrap();
-    lua.globals()
-        .set("userdata", UserData("hello".to_string()))
+    let ud = lua.create_userdata(UserData(123)).unwrap();
+    let method = lua
+        .load("function(ud, i) return ud:add(i) end")
+        .eval::<LuaFunction>()
         .unwrap();
+    let i = AtomicUsize::new(0);
 
-    c.bench_function("call async [userdata method] 10", |b| {
+    c.bench_function("userdata [async call method] 10", |b| {
         let rt = Runtime::new().unwrap();
         b.to_async(rt).iter_batched(
             || {
                 collect_gc_twice(&lua);
-                lua.load("function() for i = 1,10 do userdata:method() end end")
-                    .eval::<LuaFunction>()
-                    .unwrap()
+                (
+                    method.clone(),
+                    ud.clone(),
+                    i.fetch_add(1, Ordering::Relaxed),
+                )
             },
-            |function| async move {
-                function.call_async::<_, ()>(()).await.unwrap();
+            |(method, ud, i)| async move {
+                assert_eq!(
+                    method.call_async::<_, usize>((ud, i)).await.unwrap(),
+                    123 + i
+                );
             },
             BatchSize::SmallInput,
         );
@@ -369,27 +386,31 @@ fn call_async_userdata_method(c: &mut Criterion) {
 criterion_group! {
     name = benches;
     config = Criterion::default()
-        .sample_size(300)
+        .sample_size(500)
         .measurement_time(Duration::from_secs(10))
         .noise_threshold(0.02);
     targets =
-        create_table,
-        create_array,
-        create_string_table,
+        table_create_empty,
+        table_create_array,
+        table_create_hash,
         table_get_set,
         table_traversal_pairs,
         table_traversal_for_each,
         table_traversal_sequence,
-        create_function,
-        call_lua_function,
-        call_sum_callback,
-        call_async_sum_callback,
-        call_concat_callback,
-        create_registry_values,
-        create_userdata,
-        call_userdata_index,
-        call_userdata_method,
-        call_async_userdata_method,
+
+        function_create,
+        function_call_sum,
+        function_call_lua_sum,
+        function_call_concat,
+        function_call_lua_concat,
+        function_async_call_sum,
+
+        registry_value_create,
+
+        userdata_create,
+        userdata_call_index,
+        userdata_call_method,
+        userdata_async_call_method,
 }
 
 criterion_main!(benches);
