@@ -1,6 +1,5 @@
 use std::borrow::Cow;
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
-use std::convert::TryInto;
 use std::ffi::{CStr, CString};
 use std::hash::{BuildHasher, Hash};
 use std::os::raw::c_int;
@@ -33,6 +32,18 @@ impl<'lua> IntoLua<'lua> for Value<'lua> {
     }
 }
 
+impl<'lua> IntoLua<'lua> for &Value<'lua> {
+    #[inline]
+    fn into_lua(self, _: &'lua Lua) -> Result<Value<'lua>> {
+        Ok(self.clone())
+    }
+
+    #[inline]
+    unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
+        lua.push_value_ref(self)
+    }
+}
+
 impl<'lua> FromLua<'lua> for Value<'lua> {
     #[inline]
     fn from_lua(lua_value: Value<'lua>, _: &'lua Lua) -> Result<Self> {
@@ -55,7 +66,8 @@ impl<'lua> IntoLua<'lua> for &String<'lua> {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_ref(&self.0))
+        lua.push_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -91,7 +103,8 @@ impl<'lua> IntoLua<'lua> for &OwnedString {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_owned_ref(&self.0))
+        lua.push_owned_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -119,7 +132,8 @@ impl<'lua> IntoLua<'lua> for &Table<'lua> {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_ref(&self.0))
+        lua.push_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -156,7 +170,8 @@ impl<'lua> IntoLua<'lua> for &OwnedTable {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_owned_ref(&self.0))
+        lua.push_owned_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -184,7 +199,8 @@ impl<'lua> IntoLua<'lua> for &Function<'lua> {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_ref(&self.0))
+        lua.push_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -221,7 +237,8 @@ impl<'lua> IntoLua<'lua> for &OwnedFunction {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_owned_ref(&self.0))
+        lua.push_owned_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -249,7 +266,8 @@ impl<'lua> IntoLua<'lua> for &Thread<'lua> {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_ref(&self.0))
+        lua.push_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -286,7 +304,8 @@ impl<'lua> IntoLua<'lua> for &OwnedThread {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_owned_ref(&self.0))
+        lua.push_owned_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -314,7 +333,8 @@ impl<'lua> IntoLua<'lua> for &AnyUserData<'lua> {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_ref(&self.0))
+        lua.push_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -354,7 +374,8 @@ impl<'lua> IntoLua<'lua> for &OwnedAnyUserData {
 
     #[inline]
     unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
-        Ok(lua.push_owned_ref(&self.0))
+        lua.push_owned_ref(&self.0);
+        Ok(())
     }
 }
 
@@ -409,6 +430,18 @@ impl<'lua> FromLua<'lua> for Error {
     }
 }
 
+impl<'lua> IntoLua<'lua> for RegistryKey {
+    #[inline]
+    fn into_lua(self, lua: &'lua Lua) -> Result<Value<'lua>> {
+        lua.registry_value(&self)
+    }
+
+    #[inline]
+    unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
+        <&RegistryKey>::push_into_stack(&self, lua)
+    }
+}
+
 impl<'lua> IntoLua<'lua> for &RegistryKey {
     #[inline]
     fn into_lua(self, lua: &'lua Lua) -> Result<Value<'lua>> {
@@ -429,11 +462,24 @@ impl<'lua> IntoLua<'lua> for &RegistryKey {
     }
 }
 
+impl<'lua> FromLua<'lua> for RegistryKey {
+    #[inline]
+    fn from_lua(value: Value<'lua>, lua: &'lua Lua) -> Result<RegistryKey> {
+        lua.create_registry_value(value)
+    }
+}
+
 impl<'lua> IntoLua<'lua> for bool {
     #[inline]
     fn into_lua(self, _: &'lua Lua) -> Result<Value<'lua>> {
         Ok(Value::Boolean(self))
     }
+
+    #[inline]
+    unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
+        ffi::lua_pushboolean(lua.state(), self as c_int);
+        Ok(())
+    }
 }
 
 impl<'lua> FromLua<'lua> for bool {
@@ -445,6 +491,11 @@ impl<'lua> FromLua<'lua> for bool {
             _ => Ok(true),
         }
     }
+
+    #[inline]
+    unsafe fn from_stack(idx: c_int, lua: &'lua Lua) -> Result<Self> {
+        Ok(ffi::lua_toboolean(lua.state(), idx) != 0)
+    }
 }
 
 impl<'lua> IntoLua<'lua> for LightUserData {
@@ -633,19 +684,49 @@ impl<'lua> IntoLua<'lua> for BString {
 }
 
 impl<'lua> FromLua<'lua> for BString {
-    #[inline]
     fn from_lua(value: Value<'lua>, lua: &'lua Lua) -> Result<Self> {
         let ty = value.type_name();
-        Ok(BString::from(
-            lua.coerce_string(value)?
+        match value {
+            Value::String(s) => Ok(s.as_bytes().into()),
+            #[cfg(feature = "luau")]
+            Value::UserData(ud) if ud.1 == crate::types::SubtypeId::Buffer => unsafe {
+                let mut size = 0usize;
+                let buf = ffi::lua_tobuffer(ud.0.lua.ref_thread(), ud.0.index, &mut size);
+                mlua_assert!(!buf.is_null(), "invalid Luau buffer");
+                Ok(slice::from_raw_parts(buf as *const u8, size).into())
+            },
+            _ => Ok(lua
+                .coerce_string(value)?
                 .ok_or_else(|| Error::FromLuaConversionError {
                     from: ty,
                     to: "BString",
                     message: Some("expected string or number".to_string()),
                 })?
                 .as_bytes()
-                .to_vec(),
-        ))
+                .into()),
+        }
+    }
+
+    unsafe fn from_stack(idx: c_int, lua: &'lua Lua) -> Result<Self> {
+        let state = lua.state();
+        match ffi::lua_type(state, idx) {
+            ffi::LUA_TSTRING => {
+                let mut size = 0;
+                let data = ffi::lua_tolstring(state, idx, &mut size);
+                Ok(slice::from_raw_parts(data as *const u8, size).into())
+            }
+            #[cfg(feature = "luau")]
+            ffi::LUA_TBUFFER => {
+                let mut size = 0;
+                let buf = ffi::lua_tobuffer(state, idx, &mut size);
+                mlua_assert!(!buf.is_null(), "invalid Luau buffer");
+                Ok(slice::from_raw_parts(buf as *const u8, size).into())
+            }
+            _ => {
+                // Fallback to default
+                Self::from_lua(lua.stack_value(idx), lua)
+            }
+        }
     }
 }
 
@@ -686,6 +767,15 @@ macro_rules! lua_convert_int {
                         message: Some("out of range".to_owned()),
                     })
             }
+
+            #[inline]
+            unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
+                match cast(self) {
+                    Some(i) => ffi::lua_pushinteger(lua.state(), i),
+                    None => ffi::lua_pushnumber(lua.state(), self as ffi::lua_Number),
+                }
+                Ok(())
+            }
         }
 
         impl<'lua> FromLua<'lua> for $x {
@@ -978,6 +1068,15 @@ impl<'lua, T: IntoLua<'lua>> IntoLua<'lua> for Option<T> {
             None => Ok(Nil),
         }
     }
+
+    #[inline]
+    unsafe fn push_into_stack(self, lua: &'lua Lua) -> Result<()> {
+        match self {
+            Some(val) => val.push_into_stack(lua)?,
+            None => ffi::lua_pushnil(lua.state()),
+        }
+        Ok(())
+    }
 }
 
 impl<'lua, T: FromLua<'lua>> FromLua<'lua> for Option<T> {
@@ -988,4 +1087,13 @@ impl<'lua, T: FromLua<'lua>> FromLua<'lua> for Option<T> {
             value => Ok(Some(T::from_lua(value, lua)?)),
         }
     }
+
+    #[inline]
+    unsafe fn from_stack(idx: c_int, lua: &'lua Lua) -> Result<Self> {
+        if ffi::lua_isnil(lua.state(), idx) != 0 {
+            Ok(None)
+        } else {
+            Ok(Some(T::from_stack(idx, lua)?))
+        }
+    }
 }
