@@ -504,44 +504,6 @@ fn test_panic() -> Result<()> {
     Ok(())
 }
 
-#[test]
-fn test_result_conversions() -> Result<()> {
-    let lua = Lua::new();
-    let globals = lua.globals();
-
-    let ok = lua.create_function(|_, ()| Ok(Ok::<(), Error>(())))?;
-    let err = lua.create_function(|_, ()| Ok(Err::<(), _>("failure1".into_lua_err())))?;
-    let ok2 = lua.create_function(|_, ()| Ok(Ok::<_, Error>("!".to_owned())))?;
-    let err2 = lua.create_function(|_, ()| Ok(Err::<String, _>("failure2".into_lua_err())))?;
-
-    globals.set("ok", ok)?;
-    globals.set("ok2", ok2)?;
-    globals.set("err", err)?;
-    globals.set("err2", err2)?;
-
-    lua.load(
-        r#"
-        local r, e = ok()
-        assert(r == nil and e == nil)
-
-        local r, e = err()
-        assert(r == nil)
-        assert(tostring(e):find("failure1") ~= nil)
-
-        local r, e = ok2()
-        assert(r == "!")
-        assert(e == nil)
-
-        local r, e = err2()
-        assert(r == nil)
-        assert(tostring(e):find("failure2") ~= nil)
-    "#,
-    )
-    .exec()?;
-
-    Ok(())
-}
-
 #[test]
 fn test_num_conversion() -> Result<()> {
     let lua = Lua::new();
@@ -781,7 +743,7 @@ fn test_registry_value() -> Result<()> {
 
 #[test]
 fn test_drop_registry_value() -> Result<()> {
-    struct MyUserdata(Arc<()>);
+    struct MyUserdata(#[allow(unused)] Arc<()>);
 
     impl UserData for MyUserdata {}
 
