@@ -370,6 +370,8 @@ fn test_userdata_take() -> Result<()> {
             r => panic!("improper return for destructed userdata: {:?}", r),
         }
 
+        assert!(!userdata.is::<MyUserdata>());
+
         drop(userdata);
         lua.globals().raw_remove("userdata")?;
         lua.gc_collect()?;
@@ -400,7 +402,7 @@ fn test_userdata_take() -> Result<()> {
 
 #[test]
 fn test_userdata_destroy() -> Result<()> {
-    struct MyUserdata(Arc<()>);
+    struct MyUserdata(#[allow(unused)] Arc<()>);
 
     impl UserData for MyUserdata {}
 
@@ -951,6 +953,20 @@ fn test_userdata_method_errors() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn test_userdata_pointer() -> Result<()> {
+    let lua = Lua::new();
+
+    let ud1 = lua.create_any_userdata("hello")?;
+    let ud2 = lua.create_any_userdata("hello")?;
+
+    assert_eq!(ud1.to_pointer(), ud1.clone().to_pointer());
+    // Different userdata objects with the same value should have different pointers
+    assert_ne!(ud1.to_pointer(), ud2.to_pointer());
+
+    Ok(())
+}
+
 #[cfg(all(feature = "unstable", not(feature = "send")))]
 #[test]
 fn test_owned_userdata() -> Result<()> {
